<?php

if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

/**
 * \file acs_options.php
 * \~english
 * First file of the plugin loaded.
 * Debug options and plugin load.
 * \~french
 * Premier fichier du plugin chargé, avant même autoriser.php de la dist, mais après les fichiers de langues.
 * Options de debug et chargement du plugin.
 */

// Uncomment for debugs :
//define('_DEBUG_CRAYONS', true); // Debug plugin crayons
//define('_DEBUG_AUTORISER', true); // debug autorisations SPIP
define('_ACS_LOG', _LOG_HS); // niveau de log ACS

/*
 * Constantes de niveau de log (ACS et SPIP)
 * 0 (_LOG_HS)
 * 1 (_LOG_ALERTE_ROUGE)
 * 2 (_LOG_CRITIQUE)
 * 3 (_LOG_ERREUR)
 * 4 (_LOG_AVERTISSEMENT)
 * 5 (_LOG_INFO_IMPORTANTE)
 * 6 (_LOG_INFO)
 * 7 (_LOG_DEBUG)
 */

/*__________________________________________________________________

   Ne PAS modifier ce qui suit - Do NOT modify anything after this
	__________________________________________________________________
*/
define('_LOG_FILTRE_GRAVITE', _ACS_LOG); // définit le niveau de log SPIP

// Retourne une variable d'un plugin actif
function acs_get_from_active_plugin($plugin, $variable = false) {
	$meta_plugin = unserialize($GLOBALS['meta']['plugin']);
	$plugin = $meta_plugin[$plugin];

	if (!$variable) {
		return isset($plugin['dir']);
	}
	if (isset($plugin[$variable])) {
		return $plugin[$variable];
	}
	return false;
}

// Chemin valable espace public aussi, pas comme _DIR_PLUGIN_ACS, qui est à proscrire
define('_DIR_ACS', _DIR_PLUGINS . acs_get_from_active_plugin('ACS', 'dir') . '/');

// Dossier des paramètres et images utilisateur
// User images and parameters
// compatible mutualisation (_DIR_SITE defini)
if (defined('_DIR_SITE')) {
	$dir_site = _DIR_SITE;
	$dir_site_absolu = _DIR_RACINE ? substr(_DIR_SITE, 3) : _DIR_SITE;
}
else {
	$dir_site = _DIR_RACINE;
	$dir_site_absolu = '';
}

// Racine des dossiers du site :
define('_ACS_DIR_SITE_ROOT', $dir_site);

// Dossier du cache ACS (par defaut: tmp/cache/acs)
define('_ACS_TMP_DIR', _ACS_DIR_SITE_ROOT . _NOM_TEMPORAIRES_INACCESSIBLES . 'cache/acs/');

// Set ACS par defaut - Default ACS set
if (!acs_get('ACS_SET')) {
	acs_set('ACS_SET', 'cat');
}

// Dossier des images du set ACS actif - Active ACS set images directory
$GLOBALS['ACS_CHEMIN'] = $dir_site_absolu . _NOM_PERMANENTS_ACCESSIBLES . '_acs/' . acs_get('ACS_SET');

// Définition du dossier global des squelettes actifs d'ACS (avec override)
// Global active ACS skeletons directory definition (with override)
$dossiers_squelettes_avant_override = explode(':', $GLOBALS['dossier_squelettes']);
if (acs_get('ACS_OVER')) {
	$tas = explode(':', acs_get('ACS_OVER'));
	foreach ($tas as $dir) {
		@include(_ACS_DIR_SITE_ROOT . $dir . '/mes_options.php');
		@include(_ACS_DIR_SITE_ROOT . $dir . '/mes_fonctions.php');
		if (in_array($dir_site_absolu . $dir, $dossiers_squelettes_avant_override)) {
			continue;
		}
		if (isset($GLOBALS['dossier_squelettes']) && $GLOBALS['dossier_squelettes']) {
			$GLOBALS['dossier_squelettes'] .= ':';
		}
		$GLOBALS['dossier_squelettes'] .= $dir_site_absolu . $dir;
	}
}
if (isset($GLOBALS['dossier_squelettes']) && $GLOBALS['dossier_squelettes']) {
	$GLOBALS['dossier_squelettes'] .= ':';
}
// On ajoute le chemin du modèle cat actif
$GLOBALS['dossier_squelettes'] .= 'plugins/' . acs_get_from_active_plugin('ACS', 'dir') . '/' . 'sets/' . acs_get('ACS_SET');

// dispatch public / privé
if ((_DIR_RESTREINT != '') && ($_POST['action'] != 'poster_forum_prive')) {
	// Sauts de ligne
	$GLOBALS['spip_pipeline']['pre_propre'] .= '|post_autobr';
}

include_spip('balise/acs_balises');
include_spip('inc/acs_traduire');

acs_log('acs_options.php after includes');

/**
 * Lit, écrit, ou efface une variable
 */
function acs_get($var) {
	if (isset($GLOBALS['meta'][$var])) {
		return $GLOBALS['meta'][$var];
	}	else {
		return false;
	}
}
function acs_set($var, $value) {
	if ($value && strlen($value) > 0) {
		ecrire_meta($var, $value);
	}
	else {
		effacer_meta($var);
	}
}
function acs_unset($var) {
	acs_set($var, null);
}
/**
 * Force un recalcul des pages, CSS et scripts mis en cache ACS
 */
function acs_recalcul() {
	acs_set('ACS_MAJ', time());
	touch_meta(false); // Force la reecriture du cache SPIP des metas
}

	/**
 * Journalise les actions du plugin ACS si _ACS_LOG est supérieur à 0 dans acs_options.php
 * @param txt: texte à journaliser
 * @param gravite: cf. acs_options.php
 */
function acs_log($txt, $gravite = _LOG_HS) {
	if (_ACS_LOG > _LOG_HS) {
		spip_log($txt, 'acs', _LOG_FILTRE_GRAVITE);
	}
}
/**
 * Retourne un objet ou un tableau sous forme de tableau affichable en html
 * @param r : objet ou tableau
 * @param html : retourne du html si vrai
 * @return : objet ou tableau en mode texte ou html lisible
 */
function dbg($r, $html = false) {
	if (is_object($r) or is_array($r)) {
			ob_start();
			print_r($r);
			$r = ob_get_contents();
			ob_end_clean();
			if ($html) {
				$r = htmlentities($r);
			}
			$srch = ['/Array[\n\r]/', '/\s*[\(\)]+/', '/[\n\r]+/', '/ (?= )/s'];
			$repl = ['', '', "\n" , ($html ? '&nbsp;' : ' ')];
			$r = preg_replace($srch, $repl, $r);
			if ($html) {
				$r = nl2br($r);
			}
	}
	return $r;
}
