<?php

include_spip('inc/traduire');

// Charge la liste des classes de composants dans une globale (gain de temps, on force le calcul de composants_lire() très tôt)
if (!isset($GLOBALS['ACS_COMPOSANTS_CLASSES'])) {
	require_once _DIR_ACS . 'inc/composant/composants_lire.php';
	$GLOBALS['ACS_COMPOSANTS_CLASSES'] = serialize(array_keys(composants_lire()));
}

/**
 * Ajoute un fichier de langue de composant à la langue en cours
 *
 * @param string $c composant
 * @param string $dir (= "" ou "ecrire")
 */
function composant_ajouter_langue($c, $dir = '') {
	$lang = $GLOBALS['spip_lang'];
	if (!$lang) {
		$lang = _LANGUE_PAR_DEFAUT;
	}
	$idx = 'i18n_acs_' . $lang;
	$GLOBALS[$idx] ??= [];
	// Le mécanisme de traduction de SPIP implique que ce fichier peut être appelé à chaque traduction, donc on met en cache
	$trads = acs_cache(
		'acs_lire_langue',
		'c_' . $c . '_' . ($dir ? $dir . '_' : '') . $lang,
		[$c, $lang, $dir],
		false,
	);
	$GLOBALS[$idx] = array_merge($GLOBALS[$idx], $trads[0]);
}
/**
 * Cherche si le composant a un fichier de traductions génériques
 * définies dans lang/varacs.php, par exemple, ou dans un module de langue de SPIP)
 * et un fichier de langue dans la langue en cours ou dans la langue par défaut
 *
 * @param string $c composant
 * @param string $langfile fichier
 * @param string $lang langue
 * @param string $dir (= "" ou "ecrire")
 * @return array
 */
function acs_lire_langue($c, $lang, $dir) {
	$cla = [];
	$clpath = 'composants/' . $c . '/' . ($dir ? $dir . '/' : '') . 'lang/' . $c;
	// Cherche si le composant a un fichier de traductions génériques
	$langfile = find_in_path($clpath . ($dir ? '_' . $dir : '') . '.php');
	if ($langfile) {
		$cla = acs_lire_fichier_langue($c, $langfile);
	}
	// Cherche le fichier de langue du composant :
	$langfile = find_in_path($clpath . '_' . ($dir ? $dir . '_' : '') . $lang . '.php');
	if (!$langfile) {
		$langfile = find_in_path($clpath . '_' . ($dir ? $dir . '_' : '') . _LANGUE_PAR_DEFAUT . '.php');
		$lang = _LANGUE_PAR_DEFAUT;
	}
	if ($langfile) {
		$cla = array_merge($cla, acs_lire_fichier_langue($c, $langfile));
	}
	return $cla;
}

function acs_lire_fichier_langue($c, $langfile) {
	$cla = [];
	// Charge le fichier de langue
	$trads = lire_fichier_langue($langfile); // Appel fonction Spip
	// Affecte les traductions chargée au composant
	if ((array) $trads === $trads) {
		foreach ($trads as $k => $v) {
			$cla[$c . '_' . $k] = $v;
		}
	}
	else {
		acs_log('Error in acs_lire_fichier_langue(\'' . $c . '\', \'' . $langfile . '\'). Not a lang file.', _LOG_ERREUR);
	}
	return $cla;
}

/**
 * inc_taduire() override la fonction Spip pour ajouter les traductions de composants
 *
 * @param string $texte
 * @param string $lang
 * @return string
 */
function inc_traduire($texte, $lang) {
	if (!$texte) {
		return '';
	}
	if (substr($texte, 0, 4) !== 'acs:') {
		return inc_traduire_dist($texte, $lang);
	}
	$txt = substr($texte, 4);
	$c = get_composant($txt);
	// Traductions d'un composant
	if ($c && ($c !== $txt)) {
		if (_DIR_RESTREINT != '') {
			if (_request('action') == 'crayons_html') { // On ajoute les traductions pour les crayons
				composant_ajouter_langue($c, 'ecrire');
			}
			composant_ajouter_langue($c);
		}
		else {
			composant_ajouter_langue($c, 'ecrire');
		}
		$t = inc_traduire_dist($texte, $lang);
		if (is_trad($t, $txt)) {
			return $t;
		}
		$txt = substr($txt, strlen($c) + 1);
	}
	// On cherche une traduction générique dans lang/varacs_??.php
	$t = inc_traduire_dist('varacs:' . strtolower($txt), $lang);
	if (is_trad($t, $txt)) {
		return $t;
	}
	// On cherche une traduction dans SPIP, à défaut
	$t = inc_traduire_dist($txt, $lang);
	if (is_trad($t, $txt)) {
		return $t;
	}
	//acs_log('traduire ' . $texte . ': pas de traduction dans ' . ucfirst($c) . ' ni dans varacs pour "' . $txt .'"');
	return $txt;
}

// Teste si $t est une traduction de $txt
function is_trad($t, $txt) {
	// 3 tests, pour SPIP 2, 3, et 4
	return ($t && ($t != $txt) && ($t != str_replace('_', ' ', strtolower($txt))) && (substr($t, 0, 12) != '<blink style'));
}

function get_composant($txt) {
	static $vu = [];

	if (isset($vu[$txt])) {
		return $vu[$txt];
	}

	foreach (explode('_', $txt) as $part) {
		if (!$debut) {
			$debut = $part;
		} else {
			$debut .= '_' . $part;
		}
		if (in_array($debut, unserialize($GLOBALS['ACS_COMPOSANTS_CLASSES']))) {
			$vu[$txt] = $debut;
			return $vu[$txt];
		}
	}
}

// Compatibilité (fonction spip 4.1 inexistante en spip 3.2)
/**
 * Retourne les entrées d’un fichier de langue
 *
 * Les fichiers de langue retournent soit un array [ cle => valeur ],
 * soit peuplent une globale `$GLOBALS[$GLOBALS['idx_lang']]`.
 *
 * @return string Chemin du fichier de langue (un fichier PHP)
 * @return array<string, string>
 */
if (!function_exists('lire_fichier_langue')) {
	function lire_fichier_langue(string $fichier): array {
		$idx_lang_before = $GLOBALS['idx_lang'] ?? null;
		$idx_lang_tmp = ($GLOBALS['idx_lang'] ?? 'lang') . '@temporaire';
		$GLOBALS['idx_lang'] = $idx_lang_tmp;
		$idx_lang = include $fichier;
		$GLOBALS['idx_lang'] = $idx_lang_before;
		if (!is_array($idx_lang)) {
			if (isset($GLOBALS[$idx_lang_tmp]) and is_array($GLOBALS[$idx_lang_tmp])) {
				$idx_lang = $GLOBALS[$idx_lang_tmp];
			} else {
				$idx_lang = [];
				spip_log(sprintf('Fichier de langue incorrect : %s', $fichier), _LOG_ERREUR);
			}
			unset($GLOBALS[$idx_lang_tmp]);
		}
		return $idx_lang;
	}
}
