<?php

if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

include_spip('inc/xml');

function composants_lire() {
	static $cl = [];

	if (count($cl) > 0) {
		return $cl; // Return result if done once
	}

	require_once _DIR_ACS . 'inc/acs_cache.php';
	$set = (null !== acs_get('ACS_SET') ? acs_get('ACS_SET') : 'cat');
	$cl = acs_cache(
		'lire_composants',
		'a_' . $set . '_c',
		[$set]
	);
	$cl = $cl[0];
	return $cl;
}

/**
 * Charge en une seule fois toutes les variables de configuration des composants dans un tableau
 * mis en cache, depuis les fichiers composant.xml
 *
 * @param string $set
 * @return array
 */
function lire_composants($set) {
	// Liste des composants du modèle ACS actif
	$dirc = _DIR_ACS . 'sets/' . $set . '/composants';
	$lc = lit_composants($dirc);

	// Si override, il faut ajouter la liste des composants du ou des dossiers d'override
	if ((null !== acs_get('ACS_OVER')) && ('' !== acs_get('ACS_OVER'))) {
		$tas = explode(':', acs_get('ACS_OVER'));
		foreach ($tas as $dir) {
			$dirc = _ACS_DIR_SITE_ROOT . $dir . '/composants';
			$lc = array_merge($lc, lit_composants($dirc, $dir));
		}
	}
	ksort($lc);
	// on met le composant "fond" en premier pour permettre son override par d'autres composants
	if (array_key_exists('fond', $lc)) {
		$fond = $lc['fond'];
		unset($lc['fond']);
		$lc = array_merge(['fond' => $fond], $lc);
	}
	return $lc;
}

/*
 * Retourne la liste des composants du dossier $dirc et de leurs instances
 */
function lit_composants($dirc, $tag = '') {
	$lc = [];
	if (!(@is_dir($dirc) and @is_readable($dirc) and $d = @opendir($dirc))) {
		return $lc;
	}
	while (($f = readdir($d)) !== false && ($nb < 1000)) {
		if (($f[0] != '.') && is_dir($dirc . '/' . $f)) {
			if ($tag) {
				$lc[$f]['over'] = $tag;
			}
			$cf = $dirc . "/$f/ecrire/composant.xml";
			if (!is_file($cf) || !@is_readable($cf)) {
				$cf = find_in_path('composants/' . $f . '/ecrire/composant.xml');
			}
			if (is_file($cf) && @is_readable($cf)) {
				$config = spip_xml_load($cf); // Lit les paramètres de configuration du composant
				$c = $config['composant'][0];
				foreach (['nom', 'group', 'version'] as $cp) {
					if (isset($c[$cp][0])) {
						$lc[$f][$cp] = $c[$cp][0];
					}
				}
				// Lit les dépendances (plugins: necessite, composants: necessite_composant)
				if (spip_xml_match_nodes(',^necessite,', $config, $needs)) {
					foreach (array_keys($c) as $tag) {
						list($tag, $att) = spip_xml_decompose_tag($tag);
						if ($att) {
							$lc[$f]['necessite'][($att['set'] ? 'composant' : 'plugin')][] = $att;
						}
					}
					krsort($lc[$f]['necessite']); // on met les plugins avant les composants
				}

				// Lecture des paramètres - reading parameters
				if (is_array($c['param'])) {
					foreach ($c['param'] as $param) {
						if (is_array($param) && is_array($param['nom']) && is_array($param['valeur'])) {
							$lc[$f]['param'][$param['nom'][0]] = $param['valeur'][0];
						}
						else {
							$lc[$f]['param'][$param['nom'][0]] = true;
						}
					}
				}

				$lc[$f]['vars']['Use'] = ['type' => 'use'];
				$lc[$f]['nb_widgets'] = 0;
				$lc[$f]['cvars'] = [];
				// Lecture des variables - reading variables
				if (is_array($c['variable'])) {
					foreach ($c['variable'] as $k => $var) {
						foreach ($var as $xmltag => $value) {
							if ($xmltag === 'nom') {
								$nom = $value[0];
							}
							elseif (count($value) > 1) {
								$lc[$f]['vars'][$nom][$xmltag] = $value;
							}
							else {
								$lc[$f]['vars'][$nom][$xmltag] = $value[0];
							}
							if (($xmltag === 'type') && ($value[0] === 'widget')) {
								$lc[$f]['nb_widgets']++;
							}
							if ($xmltag == 'valeur') { // Default values
								if (substr($value[0], 0, 4) == '=acs') {
									$cvar = substr($value[0], 1);
									if (!in_array($cvar, $lc[$f]['cvars'])) {
										$lc[$f]['cvars'][] = $cvar;
									}
								}
							}
						}
					}
				}

				$instances = composant_instances($f);
				if (count($instances)) {
					foreach ($instances as $nic) {
						$lc[$f]['instances'][$nic]['on'] = acs_get('acs' . ucfirst($f) . $nic . 'Use');
					}
				}
				else {
					$lc[$f]['instances'][0]['on'] = acs_get('acs' . ucfirst($f) . 'Use');
				}
			} else {
				$lc[$f]['instances'][0]['on'] = acs_get('acs' . ucfirst($f) . 'Use');
			}
		}
		$nb++;
	}
	return $lc;
}

/**
 * Retourne un tableau de toutes les variables de composants
 *
 * Utilise le cache ACS
 */
function composants_variables() {
	static $cv = [];
	if (count($cv) > 0) {
		return $cv;
	}
	$set = (null !== acs_get('ACS_SET') ? acs_get('ACS_SET') : 'cat');
	$cv = acs_cache(
		'lecture_composants_variables',
		'a_' . $set . '_cv'
	);
	$cv = $cv[0];
	return $cv;
}
function lecture_composants_variables() {
	$cv = [];
	$composants = composants_lire();
	foreach ($composants as $c => $config) {
		$cv[$c]['vars'] = $config['vars'];
		$cv[$c]['vars']['Use'] = ['type' => 'use'];
		if ($config['over']) {
			$cv[$c]['over'] = $config['over'];
		}
	}
	return $cv;
}

function liste_variables() {
	static $lv = [];
	if (count($lv) > 0) {
		return $lv;
	}
	$cv = composants_variables();
	foreach ($cv as $c => $p) {
		foreach ($p['vars'] as $var => $vp) {
			$lv[ucfirst($c) . $var] = ['c' => $c, 'type' => $vp['type']];
		}
		foreach (composant_instances($c) as $nic) {
			foreach ($p['vars'] as $var => $vp) {
				$lv[ucfirst($c) . $nic . $var] = ['c' => $c, 'type' => $vp['type'], 'nic' => $nic];
			}
		}
	}
	return $lv;
}

function composants_liste() {
	static $cl = [];

	if (count($cl) > 0) {
		return $cl; // Return result if done once
	}

	require_once _DIR_ACS . 'inc/acs_cache.php';
	$set = (null !== acs_get('ACS_SET') ? acs_get('ACS_SET') : 'cat');
	$cl = acs_cache(
		'lecture_composants_liste',
		'a_' . $set . '_cl',
		[$set]
	);
	$cl = $cl[0];
	return $cl;
}

function lecture_composants_liste($set) {
	$cl = [];
	$composants = composants_lire();
	foreach ($composants as $c => $config) {
		if (isset($config['group'])) {
			$cl[$c]['group'] = $config['group'];
		}
		$cl[$c]['instances'] = $config['instances'];
	}
	return $cl;
}

/**
 * Retourne les instances d'un composant
 */
function composant_instances($c) {
	static $ci = [];

	if ((is_array($ci[$c])) && (count($ci[$c]) > 0)) {
		return $ci[$c];
	}

	$ci[$c] = [];
	$reg = '/acs' . ucfirst($c) . '(\d+)*Use/';
	foreach ($GLOBALS['meta'] as $meta => $val) {
		if (preg_match($reg, $meta, $matches)) {
			$ci[$c][] = $matches[1];
		}
	}
	sort($ci[$c]);
	return $ci[$c];
}

/**
 * Renvoit true si un composant a au moins une instance active
 * Le paramètre $composant est celui d'une boucle foreach (composants_liste() as $class => $composant)
 */
function composant_actif($composant) {
	$actif = false;

	// Si on upgrade depuis une vieille version, il n'y a pas d'instances de composants.
	if (!is_array($composant['instances'])) {
		return false;
	}

	foreach ($composant['instances'] as $c) {
		if ($c['on'] == 'oui') {
			$actif = true;
			break;
		}
	}
	return $actif;
}
